﻿namespace StopOrdersTest
{
	using System.Collections.Generic;
	using System.ComponentModel;
	using System.Linq;
	using System.Windows;
	using System.Windows.Threading;

	using Ecng.Common;
	using Ecng.Xaml;
	using Ecng.Trading.Algo;
	using Ecng.Trading.Algo.Strategies;
	using Ecng.Trading.BusinessEntities;
	using Ecng.Trading.Quik;

	public partial class MainWindow
	{
		class StopOrderStrategy : Strategy
		{
			public double StopLoss = 0.3;
			public double TakeProfit = 0.3;
			public double Offset = 0;
			public double Spread = 0;
			public int Slippage = 5;

			public StopOrderStrategy()
				: base()
			{
				base.Volume = 1;
			}

			protected override void OnRunning()
			{
				base.NewMyTrades += OnNewMyTrades;
				base.StopOrderChanged += OnStopOrderChanged;

				base.OnRunning();
			}

			protected override void OnStopping()
			{
				base.NewMyTrades -= OnNewMyTrades;
				base.StopOrderChanged -= OnStopOrderChanged;

				base.OnStopping();
			}

			protected override bool OnProcess()
			{
				if (base.ProcessState == StrategyProcessStates.Stopping)
				{
					base.CancelActiveOrders();

					return false;
				}

				return true;
			}

			public void MakeTrade(OrderDirections direction)
			{
				double price = base.Security.GetMarketPrice(direction, MarketPriceTypes.Following);

				Order order = base.CreateOrder(direction, price, this.Volume);

				var strategy = new MarketQuotingStrategy(order, new Unit(), new Unit());
				strategy.IsForts = true;
				strategy.Start();
				base.ChildStrategies.Add(strategy);
			}

			private void OnNewMyTrades(IEnumerable<MyTrade> trades)
			{
				Dispatcher.CurrentDispatcher.GuiSync(() => MessageBox.Show("Новых сделок {0} Поза {1}.".Put(trades.Count(), PositionManager.Position)));

				if (base.PositionManager.Position != 0)
				{
					// protect current position with stop order
					CreateTakeProfitAndStopLimit();
				}
			}

			private void OnStopOrderChanged(Order order)
			{
				Dispatcher.CurrentDispatcher.GuiSync(() => MessageBox.Show("Стоп заявка {0} изменилась на {1}.".Put(order.Id, order.State)));
			}

			private void CreateTakeProfitAndStopLimit()
			{
				MyTrade myTrade = base.MyTrades.Last<MyTrade>();
				Trade trade = myTrade.Trade;
				OrderDirections tradeDirection = myTrade.Order.Direction;

				Unit offsetUnit = new Unit(this.Offset);
				Unit spreadtUnit = new Unit(this.Spread);

				OrderDirections direction = (tradeDirection == OrderDirections.Buy ? OrderDirections.Sell : OrderDirections.Buy);
				double takeProfitPrice = trade.Price;
				double stopLossLimitPrice = trade.Price;
				double stopLossPrice = trade.Price;

				if (tradeDirection == OrderDirections.Buy)
				{
					takeProfitPrice += this.TakeProfit;
					stopLossLimitPrice -= this.StopLoss;
					stopLossPrice = stopLossLimitPrice - this.Spread;
				}
				else
				{
					takeProfitPrice -= this.TakeProfit;
					stopLossLimitPrice += this.StopLoss;
					stopLossPrice = stopLossLimitPrice + this.Spread;
				}

				Order order = base.CreateOrder(direction, stopLossPrice, trade.Volume);
				order.Type = OrderTypes.Conditional;
				order.StopCondition = new QuikStopCondition
				{
					Type = QuikStopConditionTypes.TakeProfitStopLimit,
					StopPrice = takeProfitPrice,
					StopLimitPrice = stopLossLimitPrice,
					Offset = offsetUnit,
					Spread = spreadtUnit
				};

				base.RegisterOrder(order);
			}
		}

		private StrategyManager _manager;
		private StopOrderStrategy _strategy;
		private OrderDirections _lastDir;
		private Security _lkoh;
		private Portfolio _pf;

		public MainWindow()
		{
			InitializeComponent();

			var trader = new QuikTrader { IsAsyncMode = true };
			trader.Connected += trader.StartExport;
			trader.NewSecurities += securities =>
			{
				if (_lkoh == null)
				{
					_lkoh = securities.FirstOrDefault(s => s.Code == "LKOH");

					if (_lkoh != null)
						trader.RegisterQuotes(_lkoh);
				}
			};

			trader.OrdersFailed += dict => this.GuiSync(() => MessageBox.Show(this, "Ошибка регистрации заявки."));
			trader.StopOrdersFailed += dict => this.GuiSync(() => MessageBox.Show(this, "Ошибка регистрации стоп заявки."));
			trader.ProcessDataError += error => this.GuiSync(() => MessageBox.Show(this, error.ToString()));

			trader.NewPortfolios += pfs => _pf = pfs.First();
			trader.Connect();

			_manager = new StrategyManager(trader);
		}

		protected override void OnClosing(CancelEventArgs e)
		{
			_manager.Trader.Dispose();
			_manager.Dispose();
			base.OnClosing(e);
		}

		private void Button_Click(object sender, RoutedEventArgs e)
		{
			if (_lkoh == null || _pf == null)
			{
				MessageBox.Show(this, "Нет данных.");
				return;
			}

			if (_strategy == null)
			{
				_strategy = new StopOrderStrategy();
				_manager.Register(_strategy, _pf, _lkoh);
				_strategy.Start();
			}
				
			_strategy.MakeTrade(_lastDir);
			_lastDir = _lastDir.Invert();
		}
	}
}
